<?php
if(!defined("METAL_OPTIONS_INCLUDED"))
{
	define("METAL_OPTIONS_INCLUDED",1);

/*
 * metal_options.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_options.php,v 1.27 2002/10/17 10:45:34 mlemos Exp $
 *
 */
 
class metal_options_class extends metal_base_class
{
	var $variables=array();
	var $output_directory="";
	var $base_path="";
	var $local_options="";

	Function SetupVariables(&$compiler,&$code,&$context,$path,&$variables,$array)
	{
		$definition_elements=$code->structure[$path]["Elements"];
		for($definition_element=0;$definition_element<$definition_elements;$definition_element++)
		{
			$definition_element_path=$path.",$definition_element";
			$data=$code->structure[$definition_element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "variable":
						$compiler->CopyEmptyContext($context,$variable_context);
						$variable_context->path=$definition_element_path;
						$values=array();
						$required_types=array(
							"name"=>"DATA",
							"type"=>"DATA",
							"value"=>"CONTEXT"
						);
						$optional_types=array();
						if(!$compiler->GetOptionalArgumentValues($variable_context,$values,$required_types,$optional_types,"options","definition",1))
							return(0);
						if(!$compiler->ValidateVariableName($values["name"]))
						{
							$compiler->SetElementError($context->file,$definition_element_path,"it was not defined a valid options variable name");
							return(0);
						}
						if(IsSet($variables[$values["name"]]))
						{
							$compiler->SetElementError($context->file,$definition_element_path,"it was specified an already defined options variable");
							return(0);
						}
						$variable=array();
						switch(($variable["Type"]=$values["type"]))
						{
							case "STRING":
							case "INTEGER":
							case "BOOLEAN":
							case "FLOAT":
								if(!$compiler->GetValidData($values["value"]->file,$values["value"]->path,$variable["Value"],"it was not specified a valid options variable initial value"))
									return(0);
								break;
							case "HASH":
								$variable["Value"]=array();
								if(!$this->SetupVariables($compiler,$code,$values["value"],$values["value"]->path,$variable["Value"],1))
									return(0);
								break;
							default:
								$compiler->SetElementError($values["type"]->file,$values["type"]->path,"\"".$variable["Type"]."\" is not defined a valid options variable type");
								return(0);
						}
						$variables[$values["name"]]=$variable;
						break;
					default:
						$compiler->SetElementError($context->file,$definition_element_path,$data["Tag"]." is not a supported options definition variable");
						return(0);
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($context->file,$data,$definition_element_path))
					return(0);
			}
		}
		return(1);
	}

	Function Setup(&$compiler,&$code,&$attributes)
	{
		$elements=$code->structure[$attributes->path]["Elements"];
		for($catalog_file="",$element=0;$element<$elements;$element++)
		{
			$element_path=$attributes->path.",$element";
			$data=$code->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "definition":
						if(!$this->SetupVariables($compiler,$code,$attributes,$element_path,$this->variables,0))
							return(0);
						break;
					case "output":
						if(!$compiler->GetValidData($attributes->file,$element_path,$this->output_directory,"it was not specified a valid options output directory"))
							return(0);
						break;
					case "basepath":
						if(!$compiler->GetValidData($attributes->file,$element_path,$this->base_path,"it was not specified a valid output base path"))
							return(0);
						break;
					case "local":
						$error="it was not specified a valid options local file name";
						if(!$compiler->GetValidData($attributes->file,$element_path,$this->local_options,$error))
							return(0);
						if(!strcmp($this->local_options,"")
						|| !strcmp($this->local_options,$this->object_name))
						{
							$compiler->SetElementError($attributes->file,$element_path,$error);
							return(0);
						}
						break;
					default:
						$compiler->SetElementError($attributes->file,$element_path,$data["Tag"]." is not a supported options object attribute");
						return(0);
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($attributes->file,$data,$element_path))
					return(0);
			}
		}
		if(count($this->variables)==0)
		{
			$compiler->SetElementError($attributes->file,$attributes->path,"it were not defined any options object variables");
			return(0);
		}
		if(!$compiler->CreateContextOutputDirectory($this->output_directory,$attributes,"it was not possible to create the options output directory \"".$this->output_directory."\""))
			return(0);
		return(1);
	}

	Function Execute(&$compiler,&$code,$function,&$context,&$arguments_code,&$arguments)
	{
		switch($function)
		{
			case "get":
				if(strcmp($compiler->language["name"],"PHP"))
				{
					$compiler->SetElementError($context->file,$context->path,$compiler->language["name"]." is not a language supported by the options class");
					return(0);
				}
				if($arguments_code->structure[$arguments->path]["Elements"])
				{
					if(!$compiler->GetValidName($arguments->file,$arguments->path,$variable,"it was not specified a valid options variable"))
						return(0);
				}
				else
				{
					if(!$compiler->GetValidName($context->file,$context->path,$variable,"it was not specified a valid options variable"))
						return(0);
				}
				if(!IsSet($this->variables[$variable]))
				{
					$compiler->SetElementError($context->file,$context->path,"it was requested the expression to get the value of a options variable that was not defined");
					return(0);
				}
				$context->result[]=array(
					"Data"=>array(
						"Object"=>$this->object_name,
						"Context"=>""
					),
					"Type"=>"INITIALIZATION"
				);
				$context->result[]=array(
					"Data"=>array(
						"Value"=>"\$GLOBALS[\"".$this->object_name."\"][\"$variable\"]",
						"Type"=>$this->variables[$variable]["Type"],
						"Side"=>"BOTH"
					),
					"Type"=>"EXPRESSION"
				);
				break;
			case "output":
				$output=$this->output_directory."/".$this->object_name.".php";
				$definition="<?php\n\n\$".$this->object_name."=array(\n";
				$this->BuildDefinition($this->variables,"\t",$definition);
				$definition.=");\n\n";
				$definition.="?>";
				if(!($output_file=fopen($output,"w")))
				{
					$compiler->SetElementError($context->file,$context->path,"it was not possible to create the option definition output file");
					return(0);
				}
				$written=fwrite($output_file,$definition);
				$closed=fclose($output_file);
				if($written!=strlen($definition)
				|| !$closed)
				{
					$compiler->SetElementError($context->file,$context->path,"it was not possible to write to the option definition output file");
					return(0);
				}
				$context->result[]=array(
					"Data"=>array(
						"Name"=>$output,
						"Type"=>"GENERATED"
					),
					"Type"=>"FILE"
				);
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$function." is not a supported options object function");
				return(0);
		}
		return(1);
	}

	Function BuildDefinition(&$variables,$indent,&$definition)
	{
		for($variable=0,Reset($variables);$variable<count($variables);Next($variables),$variable++)
		{
			$name=Key($variables);
			$definition.=$indent."\"$name\"=>";
			switch($variables[$name]["Type"])
			{
				case "STRING":
					$definition.="\"".$variables[$name]["Value"]."\"";
					break;
				case "INTEGER":
				case "BOOLEAN":
					$definition.=$variables[$name]["Value"];
					break;
				case "HASH":
					$definition.="array(\n";
					$this->BuildDefinition($variables[$name]["Value"],$indent."\t",$definition);
					$definition.="$indent)";
					break;
			}
			$definition.=($variable+1<count($variables) ? "," : "")."\n";
		}
	}

	Function Initialization(&$compiler,&$context,$object_context)
	{
		if(strcmp($compiler->language["name"],"PHP"))
		{
			$compiler->SetElementError($context->file,$context->path,$compiler->language["name"]." is not a language supported by the options class");
			return(0);
		}
		$relative_path=$compiler->RelativePath($this->base_path,$this->output_directory);
		$options_file=$this->object_name.".php";
		$output=$this->output_directory."/".$options_file;
		$relative_output=$relative_path."/".$options_file;
		$context->result[]=array(
			"Data"=>array(
				"Name"=>$output,
				"Type"=>"SUPPLIED"
			),
			"Type"=>"FILE"
		);
		$context->result[]=array(
			"Data"=>"require(\"$relative_output\");",
			"Type"=>"COMMAND"
		);
		if(strcmp($this->local_options,""))
		{
			if(!$compiler->ConvertData($context,"STRING",$relative_path."/".$this->local_options.".php",$local_options))
				return(0);
			$context->result[]=array(
				"Data"=>"if(file_exists($local_options))",
				"Type"=>"COMMAND"
			);
			$context->result[]=array(
				"Data"=>1,
				"Type"=>"INDENT"
			);
			$context->result[]=array(
				"Data"=>"include($local_options);",
				"Type"=>"COMMAND"
			);
			$context->result[]=array(
				"Data"=>1,
				"Type"=>"OUTDENT"
			);
		}
		return(1);
	}

};
 
}
?>